package EFS::CLI::Division;

use strict;
use warnings;
use English;

use base q{EFS::CLI};
use base q{EFS::DBI};
use Params::Validate qw( :all );

sub define {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        business    => { type => SCALAR },
        name        => { type => SCALAR },
        description => { type => SCALAR },
    });

    my $business_rs = $class->get_resultset( q{Business} );

    my $business = $business_rs->retrieve_bynames_assert(
        name => $args{business},
    );

    my $resultset = $class->get_resultset;

    my ($division) = $resultset->search({ name => $args{name} });

    if ( $division ) {

        if ( $division->business->name ne $args{business} ) {
            die(
                qq{Division $args{name} is already defined, but in },
                q{business }, $division->business->name, qq{\n},
            );
        } else {
            STDOUT->print(
                qq{Division $args{name} in },
                qq{business $args{business} is already defined\n},
            );
        }

        if ( $division->description ne $args{description} ) {
            $division->set_column( description => $args{description} );
            $division->update;
            STDOUT->print( qq{Updated description of division $args{name}\n} );
        }

    } else {
        $division = $resultset->create({
            business    => $business->id,
            name        => $args{name},
            description => $args{description},
        });
        STDOUT->print( qq{Defined division $args{name} in business $args{business}\n} );
    }

    return $division;

}

sub undef {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        business => { type => SCALAR },
        name     => { type => SCALAR },
    });

    my $resultset = $class->get_resultset;

    my $division = $resultset->retrieve_bynames_assert(
        business => $args{business},
        name     => $args{name},
    );

    $division->attrs->delete_all;
    $division->delete;

    STDOUT->print( qq{Undefined division $args{name} in business $args{business}\n} );

    return 1;

}

sub display {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        business => { type => SCALAR, optional => 1 },
        name     => { type => SCALAR, depends => [qw( business )], optional   => 1 },
    });

    my $resultset = $class->get_resultset;

    if ( not $args{business} ) {

        return $class->display_objects( objects => [ $resultset->all ] );

    } elsif ( not $args{name} ) {

        my $business_rs = $class->get_resultset( q{Business} );

        my $business = $business_rs->retrieve_bynames_assert(
            name => $args{business},
        );

        return $class->display_objects( objects => [ $business->divisions ] );

    }

    my $division = $resultset->retrieve_bynames_assert(
        business => $args{business},
        name     => $args{name},
    );

    $class->display_objects( objects => [ $division ] );

    if ( my @attrs = $division->attrs ) {
        $attrs[0]->display_objects( objects => \@attrs );
    }

    return 1;

}

sub history {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        business => { type => SCALAR },
        name     => { type => SCALAR },
        %EFS::CLI::History::validation_specs,
    });

    $args{division} = delete $args{name};

    return EFS::CLI::History->history(%args);

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
