package EFS::CLI::Install;

use strict;
use warnings;
use Try::Tiny;
use English;

use feature q{switch};

use base q{EFS::CLI};
use base q{EFS::DBI};
use base q{EFS::DistableObject};

use Params::Validate qw( :all );
use Date::Format;

sub display {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj => { type => SCALAR },
        project  => { type => SCALAR },
        release  => { type => SCALAR },
        name     => { type => SCALAR },
        force    => { type => SCALAR, optional => 1 },
    });

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    $class->display_objects( objects => [ $install ] );

    if ( my $clone = $install->cloneof ) {
        STDOUT->print(qq{\nThis install is a clone of $clone\n\n});
    }

    if ( my @clones = $install->clones ) {
        STDOUT->print(
            qq{The following installs are clones of this one:\n\t},
            join(qq{\n\t}, @clones ), qq{\n},
        );
    }

    if ( my @auditstates = sort $install->auditstates ) {
        $auditstates[0]->display_objects( objects => \@auditstates );
    }

    if ( my @attrs = sort $install->attrs ) {
        $attrs[0]->display_objects( objects => \@attrs );
    }

    return 1;

}

sub create {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj        => { type => SCALAR },
        project         => { type => SCALAR },
        release         => { type => SCALAR },
        name            => { type => SCALAR },
        nocreate        => { type => SCALAR, optional => 1 },
        force           => { type => SCALAR, optional => 1 },
        _release_locked => { type => BOOLEAN, optional => 1 },
    });

    my $release_rs = $class->get_resultset( q{Release} );

    my $release = $release_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => $args{release},
    );

    EFS::RPC->execute(
        cell   => $release->cell,
        class  => $class,
        method => q{create_rpc},
        args   => [ %args ],
    );

    my $name = $args{name};

    my $platformalias_rs = $class->get_resultset( q{PlatformAlias} );
    my $instancealias_rs = $class->get_resultset( q{PlatformInstanceAlias} );

    my ($platformalias) = $platformalias_rs->search({ name => $name });
    my ($instancealias) = $instancealias_rs->search({ name => $name });

    my $resultset = $class->get_resultset;

    if ( $platformalias ) {
        $name = $platformalias->platform->name;
    }

    if ( $instancealias ) {
        $name = $instancealias->platforminstance->name;
    }

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $name,
    );

    return $install;

}

sub create_rpc {

    my $class = shift;

    my %args = @_;

    my $history_rs = $class->get_resultset( q{History} );
    my $history = $history_rs->retrieve_assert( id => $args{history} );

    my $release_rs = $class->get_resultset( q{Release} );
    my $release = $release_rs->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        name     => $args{release},
    );

    my $name = $args{name};

    if ( $release->locked ) {
        die(
            qq{Release '$release' is locked.\n},
            qq{Installs may not be added to a locked release.\n}
        );
    }

    my $platform_rs      = $class->get_resultset( q{Platform} );
    my $instance_rs      = $class->get_resultset( q{PlatformInstance} );
    my $platformalias_rs = $class->get_resultset( q{PlatformAlias} );
    my $instancealias_rs = $class->get_resultset( q{PlatformInstanceAlias} );

    my ($platform)      = $platform_rs->search({ name => $name });
    my ($instance)      = $instance_rs->search({ name => $name });
    my ($platformalias) = $platformalias_rs->search({ name => $name });
    my ($instancealias) = $instancealias_rs->search({ name => $name });

    if ( $name ne q{common} and
             not $platform and not $instance and
                 not $platformalias and not $instancealias ) {
        die qq{No such platform(instance) or alias '$name'\n};
    }

    my $dirlock = $args{_release_locked} || EFS::DirLock->new({
        directory => $release->get_path(q{dev}),
        retry     => 1,
        command   => $history->request,
    });

    my $resultset = $class->get_resultset;

    my ($install) = $resultset->search({
        release => $release->id,
        name    => $name,
    });

    if ( not $install ) {

        if ( $platformalias ) {
            my $newname = $platformalias->platform->name;
            STDOUT->print(
                qq{Normalizing platform alias $name to $newname\n}
            );
            $name = $newname;
        }

        if ( $instancealias ) {
            my $newname = $instancealias->platforminstance->name;
            STDOUT->print(
                qq{Normalizing platform instance alias $name to $newname\n}
            );
            $name = $newname;
        }

        if ( my $object = $platform || $instance ) {
            if ( my @installs = $object->get_alias_installs($release) ) {
                if ( not $args{force} ) {
                    die(
                        q{This release has one or more alias } .
                            qq{installs for platform(instance) $object:\n\t},
                        join(qq{\n\t}, @installs ), qq{\n},
                        qq{Use --force if you want both types\n},
                    );
                }
            }
        }

        $install = $resultset->create({
            release => $release->id,
            name    => $name,
        });

    }

    if ( $args{nocreate} ) {
        STDOUT->print(qq{Defined install '$install'\n});
        return $install;
    }

    given ( $release->cell->fstype ) {

        when ( q{afs} ) {

            die qq{Unable to create installs in AFS cells yet\n};

            # XXX: Are we going to create volumes for installs?  If so, then
            # we will need to manage the names, and registering the name
            # should be done above (even if aren't creating the volume).

            # XXX: Create the volume, mount it, setacl it...

        }

        when ( q{nfs} ) {

            my $admin_path = $release->cell->admin_path( type => q{dev} );
            my $admin_dir  = qq{$admin_path/dev/$release/install};

            EFS::Util->MakeDir( path => $admin_dir );

            if ( $name eq q{common} ) {

                if ( not -d qq{$admin_dir/common} ) {
                    EFS::Util->MakeDir(
                        path  => qq{$admin_dir/common},
                        user  => $release->get_owner( type => q{user} ),
                        group => $release->get_owner( type => q{group} ),
                        mode  => $release->get_dirmode,
                    );
                }

            } else {

                EFS::Util->MakeDir( path => qq{$admin_dir/.exec} );

                my %update = (
                    command   => $history->request,
                    directory => qq{$admin_dir/.exec},
                    disable_exec_backwards =>
                        $release->find_mpr_attr(q{disable_exec_backwards}),
                    disable_exec_emulates =>
                        $release->find_mpr_attr(q{disable_exec_emulates}),
                );

                if ( -l qq{$admin_dir/.exec/$name} or not -d qq{$admin_dir/.exec/$name} ) {
                    $update{makedir} = {
                        name  => $name,
                        user  => $release->get_owner( type => q{user} ),
                        group => $release->get_owner( type => q{group} ),
                        mode  => $release->get_dirmode,
                    };
                }

                EFS::ExecLinks->update_links( %update );

            }

        }
        
    }

    my $install_dir = qq{/efs/dev/$release/install};
    $install_dir   .= $name eq q{common} ? q{/common} : qq{/.exec/$name};

    STDOUT->print(qq{_EFS_WAITFOR: $install_dir\n});
    STDOUT->print(qq{Created install $install_dir\n});

    return $install;

}

sub lock {
    return shift->_lock_unlock( @_, lockmode => q{lock} );
}

sub unlock {
    return shift->_lock_unlock( @_, lockmode => q{unlock} );
}

sub _lock_unlock {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj        => { type => SCALAR | OBJECT },
        project         => { type => SCALAR | OBJECT },
        release         => { type => SCALAR | OBJECT },
        name            => { type => SCALAR | OBJECT },
        lockmode        => { type => SCALAR },
        username        => { type => SCALAR, optional => 1 },
        verbose         => { type => BOOLEAN, optional => 1 },
        _release_locked => { type => BOOLEAN, optional => 1 },
    });

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    return EFS::RPC->execute(
        cell   => $install->project->cell,
        class  => $class,
        method => q{_lock_unlock_rpc},
        args   => [ %args ],
    );

}

sub _lock_unlock_rpc {

    my $class = shift;

    my %args = @_;

    my $history_rs = $class->get_resultset( q{History} );
    my $history = $history_rs->retrieve_assert( id => $args{history} );

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    my $install_dir = $install->get_path( q{dev} );

    my $lockmode_verb = $args{lockmode} eq q{lock} ? q{Locking} : q{Unlocking};

    my $username = $install->get_owner(
        lockmode => $args{lockmode},
        type     => q{user},
    );

    my $groupname = $install->get_owner(
        lockmode => $args{lockmode},
        type     => q{group},
    );

    if ( $args{lockmode} eq q{unlock} ) {

        if ( $args{username} and $args{username} ne $username ) {

            my $username_obj = EFS::Username->new({ name => $args{username} });

            if ( not $username_obj->is_member_of( $groupname ) ) {
                die(
                    qq{Invalid username '$args{username}'.\n},
                    qq{Must be a member of the group '$groupname'\n},
                );
            }

            $username = $args{username};

        }

    }

    STDOUT->print( qq{$lockmode_verb install $install_dir\n} );

    my $release_lock = $args{_release_locked} || EFS::DirLock->new({
        directory => $install->release->get_path(q{dev}),
        retry     => 1,
        command   => $history->request,
    });

    foreach my $clone ( $install->clones ) {
        $clone->unclone;
        STDOUT->print( qq{Install $clone is no longer a clone\n} );
    }

    given ( $install->cell->fstype ) {

        when ( q{afs} ) {
            die qq{Unable to $args{lockmode} installs in AFS yet\n};
        }

        when ( q{nfs} ) {

            my $lockgroup = $install->get_owner(
                lockmode => q{lock},
                type     => q{group},
            );

            EFS::Util->Chown(
                path          => $install_dir,
                user          => $username,
                group         => $groupname,
                lockgroup     => $lockgroup,
                verbose       => $args{verbose},
                lockmode      => $args{lockmode},
                noworldaccess => $install->get_noworldaccess,
            );

        }

    }

    return 1;

}

sub chown {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj        => { type => SCALAR | OBJECT },
        project         => { type => SCALAR | OBJECT },
        release         => { type => SCALAR | OBJECT },
        name            => { type => SCALAR | OBJECT },
        username        => { type => SCALAR, optional => 1 },
        verbose         => { type => BOOLEAN, optional => 1 },
        _release_locked => { type => BOOLEAN, optional => 1 },
    });

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    return EFS::RPC->execute(
        cell   => $install->project->cell,
        class  => $class,
        method => q{chown_rpc},
        args   => [ %args ],
    );

}

sub chown_rpc {

    my $class = shift;
    my %args  = @_;

    my $history_rs = $class->get_resultset( q{History} );
    my $history = $history_rs->retrieve_assert( id => $args{history} );

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    my $install_dir = $install->get_path(q{dev});

    my $username  = $install->get_owner( type => q{user} );
    my $groupname = $install->get_owner( type => q{group} );

    if ( $args{username} and $args{username} ne $username ) {

        my $username_obj = EFS::Username->new({ name => $args{username} });

        if ( not $username_obj->is_member_of( $groupname ) ) {
            die(
                qq{Invalid username '$args{username}'.\n},
                qq{Must be a member of the group '$groupname'\n},
            );
        }

        $username = $args{username};

    }


    my $release_lock = $args{_release_locked} || EFS::DirLock->new({
        directory => $install->release->get_path(q{dev}),
        retry     => 1,
        command   => $history->request,
    });

    given ( $install->cell->fstype ) {

        when ( q{afs} ) {
            die qq{Unable to chown installs in AFS yet\n};
        }

        when ( q{nfs} ) {

            my $lockgroup = $install->get_owner(
                lockmode => q{lock},
                type     => q{group},
            );

            my $chowned = EFS::Util->Chown(
                path          => $install_dir,
                user          => $username,
                group         => $groupname,
                lockgroup     => $lockgroup,
                verbose       => $args{verbose},
                noworldaccess => $install->get_noworldaccess,
            );

        }

    }

    return 1;

}

sub checkpoint {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj => { type     => SCALAR | OBJECT },
        project  => { type     => SCALAR | OBJECT },
        release  => { type     => SCALAR | OBJECT },
        name     => { type     => SCALAR | OBJECT },
        query    => { type     => SCALAR,
                      optional => 1 },
        nolock   => { type     => BOOLEAN,
                      optional => 1 },
        noaudit  => { type     => BOOLEAN,
                      optional => 1 },
    });

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    return EFS::RPC->execute(
        cell   => $install->project->cell,
        class  => $class,
        method => q{checkpoint_rpc},
        args   => [ %args ],
    );

}

sub checkpoint_rpc {

    my $class = shift;
    my %args  = @_;

    my $history_rs = $class->get_resultset( q{History} );
    my $history = $history_rs->retrieve_assert( id => $args{history} );

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    my $release_lock = EFS::DirLock->new({
        directory => $install->release->get_path(q{dev}),
        retry     => 1,
        command   => $history->request,
    });

    if ( $args{noaudit} && $install->release->has_prod_stages ) {
        die(
            q{Audits may not be disabled for a release which has },
            qq{a stage of prod or deprecated.\n}
        );
    }

    if ( $install->release->name eq q{incr} ) {
        $args{nolock}++;
    }

    $install->update_checkpoint(
        history => $history->id,
        nolock  => $args{nolock},
        noaudit => $args{noaudit},
    );

    return 1;

}

sub history {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj => { type => SCALAR },
        project  => { type => SCALAR },
        release  => { type => SCALAR },
        name     => { type => SCALAR },
        %EFS::CLI::History::validation_specs,
    });

    $args{install} = delete $args{name};

    return EFS::CLI::History->history(%args);

}

sub audit {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj        => { type     => SCALAR | OBJECT },
        project         => { type     => SCALAR | OBJECT },
        release         => { type     => SCALAR | OBJECT },
        name            => { type     => SCALAR | OBJECT },
        audits          => { type     => SCALAR,
                             default  => q{required},
                             optional => 1 },
        force           => { type     => BOOLEAN,
                             optional => 1 },
        verbose         => { type     => BOOLEAN,
                             optional => 1 },
        filetree        => { isa      => q{EFS::FileTree},
                             optional => 1 },
        fakereadelf     => { type     => SCALAR,
                             optional => 1 },
        _release_locked => { type     => BOOLEAN,
                             optional => 1 },
    });

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    return EFS::RPC->execute(
        cell   => $install->project->cell,
        class  => $class,
        method => q{audit_rpc},
        args   => [ %args ],
    );

}

sub audit_rpc {

    my $class = shift;
    my %args  = @_;

    my $history_rs = $class->get_resultset( q{History} );
    my $history = $history_rs->retrieve_assert( id => $args{history} );

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    my $install_dir = $install->get_path(q{dev});

    my $release_lock = $args{_release_locked} || EFS::DirLock->new({
        directory => $install->release->get_path(q{dev}),
        retry     => 1,
        command   => $history->request,
    });

    my $filetree = $args{filetree} || $install->get_filetree;

    my $max_ctime = $filetree->get_max_ctime;

    $install->set_columns({
        spaceused => $filetree->get_spaceused,
        filecount => $filetree->get_filecount,
        dircount  => $filetree->get_dircount,
        linkcount => $filetree->get_linkcount,
    });
    $install->update;

    my %failed = ();

    my %errors = ();

    my @audits = ();

    if ( $args{audits} eq q{required} ) {
        @audits = EFS::InstallAudit->get_required();
    } elsif ( $args{audits} eq q{optional} ) {
        @audits = EFS::InstallAudit->get_optional();
    } elsif ( $args{audits} eq q{all} ) {
        @audits = EFS::InstallAudit->get_all();
    } else {
        foreach my $audit ( split( m/[,\s]+/msx, $args{audits} ) ) {
            if ( EFS::InstallAudit->is_valid( $audit ) ) {
                push( @audits, $audit );
            } else {
                die qq{Invalid audit specification: '$audit'\n};
            }
        }
    }

  AUDIT: foreach my $audit ( @audits ) {

        my ($auditstate) = $install->auditstates({ name => $audit });

        if ( not $args{force} ) {

            if ( $auditstate &&
                     $auditstate->audittime >= $max_ctime &&
                         $auditstate->status ) {
                STDOUT->print(
                    qq{Skipping $audit audit for '$install', passed on },
                    time2str(q{%Y-%m-%d %H:%M:%S (GMT)},
                             $auditstate->audittime,q{GMT}),
                    qq{\n}
                );

                next AUDIT;
            }

        }

        STDOUT->print( qq{Performing $audit audit on install $install\n} );

        my @errors = ();

        my %audit_args = (
            install  => $install,
            filetree => $filetree,
        );

        if ( $audit eq q{dependencies} && $args{fakereadelf} ) {
            $audit_args{fakereadelf} = $args{fakereadelf};
        }

        my $status = 1; # assume success
        try {
            @errors = EFS::InstallAudit->$audit( %audit_args );
        }
        catch {
            $failed{$audit} = [split( m/\n+/msx, $_ )];
            $status = 0;
        };

        if ( @errors ) {
            $errors{$audit} = [@errors];
            $status = 0;
        }

        my $auditstate_rs = $class->get_resultset( q{InstallAuditState} );

        if ( not $auditstate ) {
            $auditstate = $auditstate_rs->create({
                install => $install->id,
                name    => $audit,
            });
        }

        $auditstate->set_columns({
            audittime => time,
            status    => $status,
        });

        $auditstate->update;

    }

    if ( keys %failed || keys %errors ) {

        if ( keys %failed ) {

            STDOUT->print(qq{Failed to run the following audits:\n});

            foreach my $audit ( sort keys %failed ) {
                my @fatal = @{$failed{$audit}};
                chomp(@fatal);
                STDOUT->print(
                    qq{Audit '$audit':\n\t}, join( qq{\n\t}, @fatal), qq{\n}
                );
            }

        }

        if ( keys %errors ) {

            STDOUT->print(qq{The following audits produced errors:\n});

            foreach my $audit ( sort keys %errors ) {
                my @errors = @{$errors{$audit}};
                chomp(@errors);
                STDOUT->print(
                    qq{Audit '$audit':\n\t}, join( qq{\n\t}, @errors ), qq{\n}
                );
            }

        }

        die qq{Unable to successfully complete or pass all audits\n};

    }

    return 1;

}

sub unclone {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        metaproj => { type => SCALAR },
        project  => { type => SCALAR },
        release  => { type => SCALAR },
        name     => { type => SCALAR },
    });

    my $resultset = $class->get_resultset;

    my $install = $resultset->retrieve_bynames_assert(
        metaproj => $args{metaproj},
        project  => $args{project},
        release  => $args{release},
        name     => $args{name},
    );

    $install->unclone;

    STDOUT->print(qq{Install $install is no longer a clone\n});

    return 1;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
