package EFS::CLI::PlatformInstance;

use strict;
use warnings;
use English;

use base q{EFS::CLI};
use base q{EFS::DBI};
use Params::Validate qw( :all );

sub define {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name      => { type => SCALAR },
        platform  => { type => SCALAR },
        backwards => { type => SCALAR | UNDEF, optional => 1 },
        emulates  => { type => SCALAR | UNDEF, optional => 1 },
    });

    $class->assert_platform_unused( $args{name} );

    my $platform_rs = $class->get_resultset( q{Platform} );
    my $platform =
        $platform_rs->retrieve_bynames_assert( name => $args{platform} );

    my %create = (
        platform => $platform->id,
        name     => $args{name},
    );

    my %objects = ();

    my $resultset = $class->get_resultset;

    foreach my $argument ( qw( backwards emulates ) ) {

        if ( $args{$argument} ) {

            if ( $args{name} eq $args{$argument} ) {
                die(
                    qq{Invalid $argument platform '$args{$argument}'\n},
                    qq{Matches platform instance name '$args{name}'\n},
                );
            }

            $objects{$argument} = $resultset->retrieve_bynames_assert(
                name => $args{$argument},
            );

            $create{$argument} = $objects{$argument}->id;

        }

    }

    my ($instance) = $resultset->search({ name => $args{name} });

    if ( $instance ) {

        STDOUT->print(qq{Platform instance $args{name} exists\n});

        foreach my $key ( qw( backwards emulates ) ) {

            my $new      = $objects{$key} ? $objects{$key}->id : q{};
            my $old      = $instance->$key ? $instance->$key->id : q{};
            my $new_name = $objects{$key} ? $objects{$key}->name : q{NULL};

            if ( $old ne $new ) {
                $instance->set_column( $key => $new );
                $instance->update;
                STDOUT->print(
                    qq{Updated platform instance, set $key => $new_name\n}
                );
            }

        }

    } else {
        $instance = $resultset->create( \%create );
        STDOUT->print(qq{Defined platform instance $args{name}\n});
    }

    return $instance;

}

sub undef {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR },
    });

    my $resultset = $class->get_resultset;

    my $instance = $resultset->retrieve_bynames_assert( name => $args{name} );

    if ( my @installs = $instance->installs ) {
        die(
            qq{Unable to undef platform instance '$instance',\n},
            qq{as the following installs use this name:\n\t},
            join(qq{\n\t}, map { $_->object_id } @installs ), qq{\n},
        );
    }

    if ( my @aliases = $instance->aliases ) {
        die(
            qq{Unable to undef platform instance '$instance',\n},
            qq{as it has the following aliases:\n\t},
            join(qq{\n\t}, map { $_->object_id } @aliases ), qq{\n},
        );
    }

    foreach my $type ( qw( forwards emulators ) ) {
        if ( my @depends = $instance->$type ) {
            die(
                qq{Unable to undef platform instance '$instance',\n},
                qq{as it has the following $type:\n\t},
                join( qq{\n\t}, @depends ), qq{\n},
            );
        }
    }

    $instance->delete;

    return STDOUT->print( qq{Undefined platform instance $instance\n} );

}

sub rename {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name    => { type => SCALAR },
        newname => { type => SCALAR },
    });

    $class->assert_maintenance_mode;

    my $name    = $args{name};
    my $newname = $args{newname};

    if ( $name eq $newname ) {
        die qq{Invalid arguments: new name must be different\n};
    }

    my $resultset = $class->get_resultset;
    my $instance = $resultset->retrieve_bynames_assert( name => $name );

    $class->assert_platform_unused( $newname );

    if ( $resultset->search({ name => $newname })->count ) {
        die qq{Invalid argument: $newname is already a platform instance\n};
    }

    my $instance_alias_rs = $class->get_resultset( q{PlatformInstanceAlias} );

    $instance->set_column( name => $newname );
    $instance->update;

    $instance_alias_rs->create({
        name             => $name,
        platforminstance => $instance->id,
    });

    STDOUT->print( qq{Renamed platform instance $name to $newname\n} );

    return 1;

}

sub getnames {

    my $class = shift;

    my $resultset = $class->get_resultset;

    my @instances = $resultset->all;

    STDOUT->print( map { $_->name . qq{\n} } @instances );

    return 1;

}

sub display {

    my $class = shift;

    my (%args) = validate(@_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR, optional => 1 },
    });

    my $resultset = $class->get_resultset;

    if ( not $args{name} ) {
        $class->display_objects( objects => [ $resultset->all ] );
        return 1;
    }

    my $instance = $resultset->retrieve_bynames_assert( name => $args{name} );

    $class->display_objects( objects => [ $instance ] );

    if ( my @aliases = $instance->aliases ) {
        $aliases[0]->display_objects( objects => \@aliases );
    }

    return 1;

}

sub history {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        name => { type => SCALAR },
        %EFS::CLI::History::validation_specs,
    });

    $args{platforminstance} = delete $args{name};

    return EFS::CLI::History->history(%args);

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
