package EFS::CLI::Pod;

use strict;
use warnings;
use English;

use base q{EFS::CLI};
use base q{EFS::DBI};
use Params::Validate qw( :all );
use Readonly;

sub define {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        region      => { type => SCALAR },
        campus      => { type => SCALAR },
        location    => { type => SCALAR },
        name        => { type => SCALAR },
        description => { type => SCALAR },
    });

    my $name = $args{name};

    my $location_rs = $class->get_resultset( q{Location} );

    my $location = $location_rs->retrieve_bynames_assert(
        region => $args{region},
        campus => $args{campus},
        name   => $args{location},
    );

    my $location_string = $location->object_id(q{.});

    my @fields = split m/\./msx, $name;

    Readonly my %IS_VALID_HOSTTYPE
        => map { ($_ => 1) } qw{ test dev qa uat prod };

    ##  no critic: MagicNumbers
    if ( $#fields == 4 ) {

        if ( $fields[0] ne $location->region->name ||
                 $fields[1] ne $location->campus->name ||
                     $fields[2] ne $location->name ) {
            die(
                qq{Invalid pod name '$args{name}'.\n},
                qq{The region.campus.location must match $location_string\n},
            );
        }

        if ( not $IS_VALID_HOSTTYPE{ $fields[4] } ) {
            die(
                qq{Invalid pod name '$args{name}'.\n},
                qq{The hosttype must be one of: },
                join( q{, } => keys %IS_VALID_HOSTTYPE ),
                qq{\n},
            );
        }

    } elsif ( $#fields == 1 ) {

        $name = join q{.}, $location_string, $name;

        if ( not $IS_VALID_HOSTTYPE{ $fields[1] } ) {
            die(
                qq{Invalid pod name '$args{name}'.\n},
                qq{The hosttype must be one of: },
                join( q{, } => keys %IS_VALID_HOSTTYPE ),
                qq{\n},
            );
        }

    } else {

        die(
            qq{Invalid pod name '$args{name}'.\n},
            qq{Must be 2 or 5 dot-separated fields.\n}
        );

    }

    my $resultset = $class->get_resultset;

    my ($pod) = $resultset->search({ name => $name });

    if ( $pod ) {

        STDOUT->print( qq{Pod $name already exists\n} );

        if ( $pod->description ne $args{description} ) {
            $pod->set_column( description => $args{description} );
            $pod->update;
            STDOUT->print( qq{Updated description of pod $pod\n} );
        }

    } else {

        $pod = $resultset->create({
            location    => $location->id,
            name        => $name,
            description => $args{description},
        });

        STDOUT->print( qq{Pod $name defined\n} );

    }

    return $pod;

}

sub undef {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        region   => { type => SCALAR },
        campus   => { type => SCALAR },
        location => { type => SCALAR },
        name     => { type => SCALAR },
    });

    my $resultset = $class->get_resultset;

    my $pod = $resultset->retrieve_bynames_assert( name => $args{name} );

    my $object_id = $pod->object_id;

    $pod->attrs->delete_all;
    $pod->delete;

    STDOUT->print( qq{Destroyed pod $object_id\n} );

    return 1;

}

sub display {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        region   => { type       => SCALAR,
                      optional   => 1 },
        campus   => { type       => SCALAR,
                      depends    => [qw( region )],
                      optional   => 1 },
        location => { type       => SCALAR,
                      depends    => [qw( region campus )],
                      optional   => 1 },
        name     => { type       => SCALAR,
                      optional   => 1 },
    });

    my $resultset = $class->get_resultset;

    my @pods;

    ##  no critic: CascadingIfElse
    if ( $args{name} ) {

        my $name   = $args{name};
        my @fields = split m/\./x, $name;

        if ( $#fields == 1 ) {
            if ( not $args{region} or not $args{campus} or not $args{location} ) {
                die qq{Missing required arguments: region, campus, location\n};
            }
            $name = join( q{.},
                          $args{region},
                          $args{campus},
                          $args{location},
                          $args{name} );
        }

        $pods[0] = $resultset->retrieve_bynames_assert( name => $name );

    } elsif ( $args{location} ) {

        my $location_rs = $class->get_resultset( q{Location} );

        my $location = $location_rs->retrieve_bynames_assert(
            region => $args{region},
            campus => $args{campus},
            name   => $args{location},
        );

        @pods = $location->pods;

    } elsif ( $args{campus} ) {

        my $campus_rs = $class->get_resultset( q{Campus} );

        my $campus = $campus_rs->retrieve_bynames_assert(
            region => $args{region},
            name   => $args{campus},
        );

        @pods = $campus->pods;

    } elsif ( $args{region} ) {

        my $region_rs = $class->get_resultset( q{Region} );

        my $region = $region_rs->retrieve_bynames_assert(
            name => $args{region},
        );

        @pods = $region->pods;

    } else {

        @pods = $resultset->all;

    }

    if ( not @pods ) {
        STDOUT->print( qq{No pods match the specified criteria\n} );
        return 1;
    }

    $class->display_objects( objects => \@pods );

    if ( $#pods == 0 ) {
        if ( my @attrs  = $pods[0]->attrs ) {
            $attrs[0]->display_objects( objects => \@attrs );
        }
    }

    return 1;

}

sub history {

    my $class = shift;

    my (%args) = validate( @_, {
        %EFS::CLI::validate_common,
        region   => { type     => SCALAR,
                      optional => 1 },
        campus   => { type     => SCALAR,
                      depends  => [qw( region )],
                      optional => 1 },
        location => { type     => SCALAR,
                      depends  => [qw( region campus )],
                      optional => 1 },
        name     => { type     => SCALAR,
                      optional => 1 },
        %EFS::CLI::History::validation_specs,
    });

    $args{pod} = delete $args{name};

    return EFS::CLI::History->history(%args);

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
