package EFS::Config;

use strict;
use warnings;
use English;

use IO::File;
use IO::Dir;
use Params::Validate qw( :all );
use File::Compare;
use Readonly;
use File::Slurp;

sub update_file {

    my $class = shift;

    my (%args) = validate( @_, {
        file    => { type     => SCALAR },
        content => { type     => SCALAR },
        tmpfile => { type     => SCALAR,
                     optional => 1 },
        message => { type     => SCALAR,
                     optional => 1 },
    });

    my $file    = $args{file};
    my $content = $args{content};
    my $tmpfile = $args{tmpfile} || qq{$file.$PID};
    my $message = $args{message} || qq{Updated file $file\n};

    write_file( $tmpfile, $content );

    my $changes = -f $file ? compare( $file, $tmpfile ) : 1;

    if ( $changes ) {
        rename( $tmpfile, $file ) ||
            die qq{Unable to rename $tmpfile to $file: $ERRNO\n};
        STDOUT->print($message);
    } else {
        unlink( $tmpfile ) ||
            die qq{Unable to unlink $file: $ERRNO\n};
    }

    return $changes;

}

sub parse_efsservers {

    my $class = shift;

    my %servers;

    my $servers_file =
        q{/efs/dist/efs/config-efsserver/current/common/etc/efsserver.txt};

    my $content = read_file( $servers_file );

    foreach my $line ( split m{\n+}msx, $content ) {

        chomp $line;
        $line =~ s{\# .* $}{}msx;
        $line =~ s{^ \s*}{}msx;
        $line =~ s{\s+ $}{}msx;
        next if not $line;
        $line = lc $line;

        my ($cell, $servertype, $fqdn) = split m{\s+}msx, $line;

        $servertype ||= q{prod};
        $servers{$cell}->{$servertype}->{$fqdn}++;

    }

    return %servers;

}

#
# XXX: This has to go.  The servertype should be specified on the CLI,
# or yanked out of efsd.conf
#
sub parse_servertype {

    my $class = shift;
    my %args  = @_;

    my $release = q{};

    if ( defined $FindBin::Bin &&
             $FindBin::Bin =~ m{/efs/dist/efs/core/([^/]+)/}msx ) {
        $release = $1;
    } else {
        # Fall back on the client's hosttype.
        # This is essential for the test machines
        my $config = EFS::Config::Client->new;
        $release = $config->get_value( 'client', 'hosttype' );
    }

    if ( $release ~~ [qw( dev test prod )] ) {
        return $release;
    }

    my $dirname = q{/efs/dist/efs/core};

    my $project = IO::Dir->new($dirname) ||
        die qq{Unable to opendir $dirname: $ERRNO\n};

    my %release_of = (
        prod => q{},
        dev  => q{},
    );

    foreach my $dirent ( $project->read ) {

        if ( $dirent eq q{.} || $dirent eq q{..} ) {
            next;
        }

        if ( not -l qq{$dirname/$dirent} ) {
            next;
        }

        my $target = readlink( qq{$dirname/$dirent} ) ||
            die qq{Unable to readlink $dirname/$dirent: $ERRNO\n};

        $release_of{$dirent} = $target;

    }

    if ( $release_of{prod} eq $release ) {
        return 'prod';
    } elsif ( $release_of{dev} eq $release ) {
        warn qq{efs/core/$release will connect to dev EFS servers\n};
        return q{dev};
    } else {
        my $message =
            qq{Unable to determine default servertype of release '$release'\n};
        if ( $args{fatal} ) {
            die $message;
        } else {
            warn( $message, qq{Defaulting to 'dev'.\n} );
            return q{dev};
        }
    }

    return;

}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
