package EFS::DirLock;

use strict;
use warnings;
use English;

use EFS::FQDN qw(fqdn);

use Moose;
use Params::Validate qw( :all );
use File::Temp;
use Regexp::Common;
use Errno;
use Readonly;

has q{directory} => (
    is       => q{ro},
    isa      => q{Str},
    required => 1,
);

has q{alternate} => (
    is      => q{rw},
    isa     => q{Str},
    clearer => q{_clear_alternate},
);

has q{_lockfile} => (
    is        => q{rw},
    isa       => q{Str},
    predicate => q{_has_lockfile},
    clearer   => q{_clear_lockfile},
);

has q{_pid} => (
    is      => q{rw},
    isa     => q{Int},
    default => sub { return $PID },
);

{

    sub BUILD {

        my ($self,$args_ref) = @_;

        # remove args handled by Moose, as they've already been checked
        for my $attr ( qw(directory alternate) ) {
            delete $args_ref->{$attr};
        }

        my @args = %{ $args_ref };

        my (%args) = validate( @args, {

            command    => { type     => SCALAR },

            lockname   => { type     => SCALAR,
                            default  => q{.lock},
                            optional => 1 },
            retrycount => { type     => SCALAR,
                            default  => 10,
                            optional => 1 },
            retrysleep => { type     => SCALAR,
                            default  => 60,
                            optional => 1 },
            retry      => { type     => BOOLEAN | SCALAR,
                            optional => 1 },
            stalelock  => { type     => BOOLEAN | SCALAR,
                            optional => 1 },

        });

        if ( my $altdir = $self->alternate ) {
            # When set at construction time, we take just a dirname
            $self->alternate($altdir . qq{/$args{lockname}});
        }

        my $retrycount_init = $args{retrycount};
        Readonly my $MAX_RETRY => 3;

        my $lockfile = $self->directory .qq{/$args{lockname}};

        my $fqdn = fqdn;

        if ( $args{stalelock} && -f $lockfile ) {
            unlink($lockfile) ||
                die qq{Unable to unlink stale lockfile $lockfile: $ERRNO\n};
        }

        my $tmpfh = File::Temp->new(
            TEMPLATE => q{.efs_lock_XXXXXXXX},
            DIR      => $self->directory,
            UNLINK   => 1,
        ) || die qq{Cannot open the temporary lock file\n};

        my $tmpname = $tmpfh->filename;

      LOCK: {

            if ( link( $tmpname, $lockfile ) ) {
                $self->_lockfile($lockfile);
                last LOCK;
            }

            if ( $ERRNO != Errno::EEXIST ) {
                die qq{Unable to create lockfile $lockfile: $ERRNO\n};
            }

            my $lockfh = IO::File->new( $lockfile ) || do {
                if ( $ERRNO == Errno::ENOENT ) {
                    redo LOCK;
                } else {
                    die qq{Unable to open lockfile $lockfile: $ERRNO\n};
                }
            };

            my ($lockhost,$lockpid,$lockcommand) =
                qw( unknown unknown unknown );

            ($lockhost,$lockpid,$lockcommand) =
                split( qr/ : /msx, $lockfh->getline );

            chomp($lockcommand);

            my $efsserver_rs = EFS::DBI->get_resultset( q{EFSServer} );

            my ($efsserver) = $efsserver_rs->search({ name => $lockhost });

            my $stale = 0;

            if ( $efsserver ) {

                $stale = EFS::RPC->execute_on(
                    efsservers => [ $efsserver ],
                    class      => q{EFS::DirLock},
                    method     => q{stalelock},
                    args       => [
                        lockpid => $lockpid,
                    ],
                );

                if ( $stale ) {
                    unlink($lockfile) ||
                        die qq{Unable to unlink stale $lockfile: ERRNO\n};
                }

            } else {

                # If the hostname is NOT a valid EFSServer, then let's
                # assume the lock is stale

                STDOUT->print(
                    qq{No such efsserver $lockhost\n},
                    qq{Unlinking stale lockfile and retrying...\n},
                );

                unlink($lockfile) ||
                    die qq{Unable to unlink stale $lockfile: ERRNO\n};

            }

            if ( not -f $lockfile ) {
                redo LOCK;
            }

            # Eventually, let's remove very old locks (several
            # days?), but for now, we want to have a chance to
            # investigate these, and make sure they are the result
            # of crashes, and not EFS bugs.

            last LOCK unless $args{retry};

            last LOCK unless $args{retrycount}--;

            if ( $retrycount_init - $args{retrycount} > $MAX_RETRY ) {
                STDOUT->print(
                    $self->directory, qq{ is already locked,\n},
                    qq{Hostname: $lockhost, PID: $lockpid, },
                    qq{Command: $lockcommand\n},
                    qq{Sleeping $args{retrysleep} and retrying... },
                    qq{($args{retrycount} attempts remaining)\n}
                );
            }

            sleep $args{retrysleep};

            redo LOCK;

        }

        if ( not $self->_has_lockfile ) {
            die( q{Unable to lock }, $self->directory, qq{\n} );
        }

        # At this point, we have locked the directory

        $tmpfh->print( join( q{:}, $fqdn, $PID, $args{command}), qq{\n} ) ||
            die qq{Unable to write to $lockfile: $ERRNO\n};

        $tmpfh->close ||
            die qq{Unable to close $lockfile: $ERRNO\n};

        unlink($tmpname) ||
            die qq{Unable to unlink $tmpname: $ERRNO\n};

        return;
    }

    sub destroy {

        my $self = shift;

        my $lockfile;
        if ( $lockfile = $self->_lockfile and -f $lockfile ) {
            unlink($lockfile) ||
                die qq{Unable to unlink lockfile $lockfile: $ERRNO\n};
            $self->_clear_lockfile;
        }

        my $altfile;
        if ( $altfile = $self->alternate and -f $altfile ) {
            unlink($altfile) ||
                die qq{Unable to unlink lockfile $altfile: $ERRNO\n};
            $self->_clear_alternate;
        }

        return 1;

    }

    sub DEMOLISH {

        my $self = shift;

        # Don't let child processes clean us up...
        return unless $PID == $self->_pid;

        return $self->destroy;

    }

}

sub stalelock {

    my $class = shift;

    my (%args) = validate( @_, {
        lockpid => { type => SCALAR },
    });

    my $lockpid = $args{lockpid};

    Readonly my $BOGUS_PID => -1;

    my $stale = 0;

    if ( $lockpid == $BOGUS_PID ) {
        # This is used by the test suite to simulate a bogus pid.
        $stale = 1;
    } elsif ( not kill 0, $lockpid ) {
        $stale = 1;
    }

    return $stale;

}

no Moose;
__PACKAGE__->meta->make_immutable;
1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
