package EFS::Notification;

use strict;
use warnings;
use English qw(-no_match_vars);

use List::MoreUtils qw( uniq );
use Mail::Sender;
use Params::Validate qw( :all );
use FindBin;
use Text::Template;


### Class methods

sub _from_address {
    my $parameter_rs = EFS::DBI->get_resultset(q{Parameter});
    return $parameter_rs->configured(q{notification_from_address});
}

sub _smtp_server {
    my $parameter_rs = EFS::DBI->get_resultset(q{Parameter});
    return $parameter_rs->configured(q{notification_smtp_server});
}

sub _email_domain {
    my $parameter_rs = EFS::DBI->get_resultset(q{Parameter});
    return $parameter_rs->configured(q{notification_email_domain});
}

sub username_to_email {
    my ($class,$username) = @_;
    return $username . q{@} . $class->_email_domain;
}

my %common_notify_args = (
    reply_to            => { type  => SCALAR | OBJECT,    optional => 1, },
    recipients          => { type  => ARRAYREF },
    priority            => { regex => qr/\A [1-5] \z/xms, default  => 3, },
    send_to_owner_list  => { type  => BOOLEAN,            default  => 1, },
    send_to_notify_list => { type  => BOOLEAN,            default  => 1, },
    authuser            => 1,
);

sub notify_mprs_template {

    my $class = shift;
    my @args  = @_;

    my (%args) = validate_with(
        params => \@args,
        spec => {
            template => 1,
            template_args => { type => HASHREF, default => {}, },
            %common_notify_args,
        },
        allow_extra => 1
    );

    # XXX: Need a better mechanism for finding the templates dir
    my $template_file = qq{$FindBin::Bin/../templates/$args{template}.txt};

    if ( not -f $template_file ) {
        die qq{No such template file $template_file\n};
    }

    my $subject;

    my $template = Text::Template->new( SOURCE => $template_file ) or
        die(
            qq{Unable to create template from $template_file:\n},
            $Text::Template::ERROR,
        );

    my $body = $template->fill_in(
        HASH => { subject => \$subject, %{ $args{template_args} } }
    );

    if ( not $body ) {
        die(
            qq{Unable to fill in template $template_file:\n},
            $Text::Template::ERROR,
        );
    }

    return $class->notify_mprs(
        %args,
        subject => $subject,
        body    => $body,
    );

}

sub notify_mprs {

    my $class = shift;
    my @args  = @_;

    my (%args) = validate_with(
        params      => \@args,
        spec        => { subject => 1, body => 1, %common_notify_args, },
        allow_extra => 1
    );

    my @list_fields;
    if ( $args{send_to_owner_list} ) {
        push @list_fields, q{owner_list};
    }
    if ( $args{send_to_notify_list} ) {
        push @list_fields, q{notify_list};
    }

    # We'll accept reply_to as either an object or an address. If we
    # get an object, if we get nothing, use the username.  Otherwise
    # use whatever we were passed (a string address)

    if ( not exists $args{reply_to} ) {
        $args{reply_to} = $class->username_to_email( $args{authuser} );
    }
    elsif ( ref $args{reply_to} ) {
        $args{reply_to} = $args{reply_to}->mp_owner_list;
    }

    my @addrs = $class->username_to_email( $args{authuser} );
    for my $recip ( @{ $args{recipients} } ) {
        push @addrs,
            ref $recip ? $recip->all_mpr_fields(@list_fields) : $recip;
    }
    @addrs = uniq sort @addrs;

    return $class->_send_email(addrs => \@addrs, %args);
}

sub _send_email {
    my ( $class, %args ) = @_;

    return unless @{ $args{addrs} };

    my $from_address = $class->_from_address;

    my @priority = qw( None Highest High Normal Low Lowest );

    if ( $ENV{EFS_TEST_SUITE} ) {

        STDOUT->print(
            qq{TEST: The following mail message would have been generated:\n},
            qq{TEST: From: $from_address\n},
            (
                q{TEST: To: }, ( join q{, }, @{ $args{addrs} } ),
                qq{\n}
            ),
            qq{TEST: Cc: $from_address\n},
            (
                q{TEST: Reply-To: }, $args{reply_to}, qq{\n}
            ),
            q{TEST: Priority: },
            $priority[ $args{priority} + 0 ], qq{\n},
            q{TEST: Subject: }, $args{subject}, qq{\n\n},
        );

        foreach my $line ( split qr/ \n /msx, $args{body} ) {
            STDOUT->print(qq{TEST: $line\n});
        }

        return 1 unless $ENV{EFS_TEST_SEND_MAIL};

    }

    my $sender = Mail::Sender->new({
        smtp      => $class->_smtp_server,
        from      => $from_address,
        fake_from => qq{EFS Notifications <$from_address>},
        replyto   => $args{reply_to},
        priority  => $args{priority},
        on_errors => q{undef},
    }) or die(q{Couldn't get Mail::Sender object: },
              $Mail::Sender::Error);

    $sender->OpenMultipart({
        to      => \@{ $args{addrs} },
        cc      => $from_address,
        subject => $args{subject},
    }) or die(q{Couldn't prepare email notification: },
              $sender->{error_msg});

    $sender->Body({ msg => $args{body} }) or
        die(q{Couldn't prepare email notification body: },
            $sender->{error_msg});

    $sender->Close or
        die(q{Couldn't send email notification: },
            $sender->{error_msg});

    return;
}

1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
