package EFS::Package;

use strict;
use warnings;
use English;

use Moose;
use Date::Calc qw(:all);
use Params::Validate qw( :all );
use File::Basename qw( dirname fileparse );
use File::Slurp;
use File::Temp;
use IO::Dir;
use LWP::UserAgent;

has q{force} => ( is => q{ro}, isa => q{Int}, default => 0 );

has q{upload_server}   => ( is => q{rw}, isa => q{Str} );
has q{upload_username} => ( is => q{rw}, isa => q{Str} );
has q{upload_root}     => ( is => q{rw}, isa => q{Str} );
has q{upload_sshkey}   => ( is => q{rw}, isa => q{Str} );
has q{upload_hook}     => ( is => q{rw}, isa => q{Str} );
has q{upload_content}  => ( is => q{rw}, isa => q{HashRef} );
has q{upload_depends}  => ( is => q{rw}, isa => q{HashRef} );

has q{download_url}     => ( is => q{rw}, isa => q{Str} );
has q{download_content} => ( is => q{rw}, isa => q{HashRef} );
has q{download_depends} => ( is => q{rw}, isa => q{HashRef} );

has q{history}  => ( is => q{ro}, isa => q{Int} );
has q{metaproj} => ( is => q{ro}, isa => q{Str} );
has q{project}  => ( is => q{ro}, isa => q{Str} );
has q{name}     => ( is => q{ro}, isa => q{Str} );
has q{mpr}      => ( is => q{rw}, isa => q{Str} );

has q{cells}    => ( is => q{ro}, isa => q{ArrayRef} );

has q{_metaproj_obj} => ( is => q{rw}, isa => q{Object} );
has q{_project_obj}  => ( is => q{rw}, isa => q{Object} );
has q{_release_obj}  => ( is => q{rw}, isa => q{Object} );

has q{_metadata} => ( is => q{rw}, isa => q{Object} );

has q{_lwp_ua} => (
    is => q{ro},
    isa => q{LWP::UserAgent},
    default => sub { LWP::UserAgent->new( env_proxy => 1 ) },
);

sub BUILD {

    my $self = shift;

    $self->mpr( join q{/}, $self->metaproj, $self->project, $self->name );

    my $metaproj_rs = EFS::DBI->get_resultset( q{Metaproj} );
    my $metaproj_obj = $metaproj_rs->retrieve_bynames_assert(
        name => $self->metaproj,
    );
    $self->_metaproj_obj( $metaproj_obj );

    my $object = $metaproj_obj;

    my $project_rs = EFS::DBI->get_resultset( q{Project} );
    my ($project_obj) = $project_rs->retrieve_bynames(
        metaproj => $self->metaproj,
        name     => $self->project,
    );

    if ( $project_obj ) {
        $self->_project_obj( $project_obj );
        $object = $project_obj;
    }

    foreach my $attr ( qw( upload_server upload_username
                           upload_root upload_sshkey
                           upload_hook download_url ) ) {
        $self->$attr( $object->find_mpr_attr( $attr ) || q{} );
    }

    foreach my $attr ( qw( upload_content download_content ) ) {
        my $value = $object->find_mpr_attr( $attr ) || q{install};
        $value = q{install,build,src} if $value eq q{all};
        $self->$attr( { map { $_ => 1 } split m{,}msx, $value } );
    }

    foreach my $attr ( qw( upload_depends download_depends ) ) {
        my $value = $object->find_mpr_attr( $attr ) || q{runtime};
        $value = q{runtime,build,test} if $value eq q{all};
        $self->$attr( { map { $_ => 1 } split m{,}msx, $value } );
    }

    my $release_rs = EFS::DBI->get_resultset( q{Release} );
    my $release_obj = $release_rs->retrieve_release_or_alias(
        metaproj => $self->metaproj,
        project  => $self->project,
        name     => $self->name,
    );

    return if not $release_obj;

    # Need to rethink this

    if ( $release_obj->isa( q{EFS::DBI::Result::ReleaseAlias} ) ) {
        $self->_release_obj( $release_obj->target_release );
    }
    else {
        $self->_release_obj( $release_obj );
    }

    return 1;

}

sub package {

    my $self = shift;

    my $mpr     = $self->mpr;
    my $release = $self->_release_obj || die qq{No such release '$mpr'\n};

    my $releasealiases = $self->releasealiases;

    $release->assert_uploadable;

    if ( $release->downloaded ) {
        if ( $self->force ) {
            $release->set_column( downloaded => undef );
            $release->update;
        } else {
            STDOUT->print(
                qq{Release $release was downloaded on }, $release->downloaded, qq{\n},
                qq{Use --force to re-package\n},
            );
            return 1;
        }
    }

    if ( $release->packaged  ) {
        if ( $self->force ) {
            $release->set_column( packaged => undef );
            $release->update;
        } else {
            STDOUT->print(
                qq{Release $release was packaged on }, $release->packaged, qq{\n},
                qq{Use --force to re-package\n},
            );
            return 1;
        }
    }

    $release->validate_checkpoint;

    if ( not $release->locked ) {
        die( qq{Unable to package unlocked release $release\n} );
    }

    my $upload  = $release->get_path(q{dev}) . q{/build/upload};
    my $package = qq{$upload/package};
    my $aliases = qq{$upload/aliases};

    EFS::Util->RemoveDirHierarchy( top => $upload );

    EFS::Util->MakeDir( path => $upload );
    EFS::Util->MakeDir( path => $package );

    my %sum_of = ();

    my $upload_content = $self->upload_content;

    my @installs = sort $release->installs;

    if ( $upload_content->{install} ) {

        EFS::Util->MakeDir( path => qq{$package/install} );

        foreach my $install ( @installs ) {

            my $name    = $install->name;
            my $tarball = qq{$package/install/$name.tar.bz2};

            $self->package_dir(
                dir  => $install->get_path( q{dev} ),
                file => $tarball,
            );

            $sum_of{ qq{install/$name} } = $self->sha1sum($tarball);

        }

    }

    if ( $upload_content->{build} ) {

        EFS::Util->MakeDir( path => qq{$package/build} );

        # NOTE: The "common" part of the build is always packaged
        # so we include it in case there's no common install.
        my @execs = grep { $_ ne q{common} } map { $_->name } @installs;

        foreach my $name ( @execs, q{common} ) {

            my $tarball = qq{$package/build/$name.tar.bz2};

            # XXX: This is a one-off, migration hack, to deal with
            # changes in efsdeploy needs for remote builds.  Hard to explain...
            if ( $name eq q{common} ) {
                $self->_migrate_build_common( $release );
            }

            my $build_dir = $release->get_path( q{dev} ) . qq{/build/$name};

            # XXX: Not all release have these directories, but we make too
            # many assumptions about their existence, so assert them.
            EFS::Util->MakeDir( path => $build_dir );

            $self->package_dir(
                dir  => $build_dir,
                file => $tarball,
            );

            $sum_of{ qq{build/$name} } = $self->sha1sum($tarball);

        }

    }

    if ( $upload_content->{src} ) {

        my $tarball = qq{$package/src.tar.bz2};

        $self->package_dir(
            dir  => $release->get_path( q{dev} ) . q{/src},
            file => $tarball,
        );

        $sum_of{ q{src} } = $self->sha1sum($tarball);

    }

    # XXX: Should we chdir back to wherever we were before?

    # Create build/package/metadata.conf

    my $sp4 = q{ } x 4;
    my $sp8 = q{ } x 8;

    my @names  = map { $_->name } @installs;
    my $global = join( qq{ \\\n}, map { qq{$sp8$_} } @names );

    my $sums   = join( qq{\n}, map { qq{$sp4$_ = $sum_of{$_}} } sort keys %sum_of );

    my $metaproj_name = $self->metaproj;
    my $project_name  = $self->project;
    my $release_name  = $release->name;

    my $metadata = <<"METADATA";
[global]
    metaproj = $metaproj_name
    project  = $project_name
    release  = $release_name
    installs = \\
$global

[sums]
$sums

METADATA

    my $depends = {};

    foreach my $type ( qw( runtime build test ) ) {
        my @depends       = sort $release->dependencies( type => $type );
        next if not @depends;
        my @mprs          = map { $_->dependency_object_id } @depends;
        $depends->{$type} = join( qq{ \\\n}, map { qq{$sp8$_} } @mprs );
    }

    if ( my @types = sort keys %{ $depends } ) {
        $metadata .= <<"METADATA";
[depends]
METADATA
        foreach my $type ( @types ) {
            $metadata .= <<"METADATA";
    $type = \\
$depends->{$type}
METADATA
        }

    }

    # http://trac.openefs.org/ticket/158
    # This entire section goes away.  Release will no longer include the
    # releasealiases.

    if ( keys %{ $releasealiases } ) {

        $metadata .= qq{\n[releasealiases]\n};
        $metadata .= join(
            q{},
            map { qq{$sp4$_ = $releasealiases->{$_}\n} }
                sort keys %{ $releasealiases }
        );

        EFS::Util->MakeDir( path => $aliases );

        foreach my $name ( sort keys %{ $releasealiases } ) {
            EFS::Util->MakeLink(
                from => qq{$aliases/$name},
                to   => $releasealiases->{$name},
            );
        }

    }

    write_file( qq{$package/metadata.conf}, $metadata );

    $release->set_column( packaged => join(q{-}, Today ) );
    $release->set_column( uploaded => undef );
    $release->update;

    STDOUT->print( qq{Packaged $release\n} );

    return 1;

}

sub upload {

    my $self = shift;

    my $metaproj_name = $self->metaproj;
    my $project_name  = $self->project;
    my $mpr           = $self->mpr;
    my $release       = $self->_release_obj || die qq{No such release '$mpr'\n};
    my $release_name  = $release->name;

    $release->assert_uploadable;

    if ( $release->downloaded ) {
        if ( $self->force ) {
            $release->set_column( downloaded => undef );
            $release->update;
        } else {
            STDOUT->print(
                qq{Release $release was downloaded on }, $release->downloaded, qq{\n},
                qq{Use --force to upload\n},
            );
            return 1;
        }
    }

    if ( $release->uploaded ) {
        if ( $self->force ) {
            $release->set_column( uploaded => undef );
            $release->update;
        }
        else {
            STDOUT->print(
                qq{Release $release was uploaded on }, $release->uploaded, qq{\n},
                qq{Use --force to re-upload\n},
            );
            return 1;
        }
    }

    my $release_dir = $release->get_path(q{dev});

    my $upload_root   = $self->upload_root;
    my $upload_target = qq{$upload_root/$metaproj_name/$project_name/$release_name};

    $self->remote_mkdir( dir => dirname $upload_target );

    $self->rsync(
        from   => qq{$release_dir/build/upload/package/},
        to     => $upload_target,
        delete => 1,
    );

    if ( keys %{ $self->releasealiases } ) {
        $self->rsync(
            from => qq{$release_dir/build/upload/aliases/},
            to   => dirname $upload_target,
        );
    }

    $self->run_upload_hook;

    my $upload_server = $self->upload_server;
    STDOUT->print( qq{Uploaded $release package to $upload_server\n} );

    $release->set_column( uploaded => join(q{-}, Today ) );
    $release->update;

    return 1;

}

sub download {

    my $self = shift;

    my $metadata = $self->_metadata;

    my $metaproj_name = $metadata->{global}->{metaproj};
    my $project_name  = $metadata->{global}->{project};
    my $release_name  = $metadata->{global}->{release};

    my $metaproj_rs = EFS::DBI->get_resultset( q{Metaproj} );
    my $metaproj = $metaproj_rs->retrieve_bynames_assert(
        name => $metadata->{global}->{metaproj},
    );

    my $project_rs = EFS::DBI->get_resultset( q{Project} );
    my ($project) = $project_rs->search({
        metaproj => $metaproj->id,
        name     => $project_name,
    });

    if ( $project ) {
        $project->assert_downloadable;
    }
    else {
        $metaproj->assert_downloadable;
        $project = EFS::CLI::Project->create(
            metaproj => $metaproj_name,
            name     => $project_name,
        );
    }

    my $release_rs = EFS::DBI->get_resultset( q{Release} );
    my ($release) = $release_rs->search({
        project  => $project->id,
        name     => $release_name,
    });

    if ( $release ) {

        if ( $release->downloaded ) {
            if ( $self->force ) {
                $release->set_column( downloaded => undef );
                $release->update;
            }
            else {
                STDOUT->print(
                    qq{Release $release was downloaded on },
                    $release->downloaded, qq{\n},
                    qq{Use --force to re-download\n},
                );
                return 1;
            }
        }

        if ( $release->locked ) {
            # XXX: If --force, then unlock
            die(
                qq{Release $release is locked\n},
                qq{Unable to download a locked release\n},
            );
        }

    }
    else {

        $release = EFS::CLI::Release->create_rpc(
            history  => $self->history,
            metaproj => $metaproj_name,
            project  => $project_name,
            name     => $release_name,
        );

    }

    my $download_content = $self->download_content;
    my $download_content_types = join( q{,}, sort keys %{ $download_content } );
    STDOUT->print( qq{Downloading $download_content_types for $release\n} );

    my $download = $release->get_path( q{dev} ) . q{/build/download};

    EFS::Util->MakeDir( path => $download );
    EFS::Util->MakeDir( path => qq{$download/install} );
    EFS::Util->MakeDir( path => qq{$download/build} );

    my $download_url  = $self->download_url;
    my $download_root = qq{$download_url/$metaproj_name/$project_name/$release_name};

    $self->download_file(
        from => qq{$download_root/metadata.conf},
        to   => qq{$download/metadata.conf},
    );

    if ( $download_content->{src} ) {

        my $file = q{src.tar.bz2};

        $self->download_file(
            from => qq{$download_root/$file},
            to   => qq{$download/$file},
            sum  => $metadata->{sums}->{src},
        );

        $self->extract_file(
            file => qq{$download/$file},
            dir  => $release->get_path( q{dev} ),
        );

    }

    my @installs = $self->filter_installs( split m{\s+}msx, $metadata->{global}->{installs} );

    if ( $download_content->{install} and @installs ) {

        my $download_installs = join( q{,}, @installs );
        STDOUT->print( qq{Downloading installs: $download_installs\n} );

        foreach my $install_name ( @installs ) {

            my $file = qq{$install_name.tar.bz2};

            $self->download_file(
                from => qq{$download_root/install/$file},
                to   => qq{$download/install/$file},
                sum  => $metadata->{sums}->{ qq{install/$install_name} },
            );

            my $install = EFS::CLI::Install->create_rpc(
                history  => $self->history,
                metaproj => $metaproj_name,
                project  => $project_name,
                release  => $release_name,
                name     => $install_name,
            );

            $self->extract_file(
                file => qq{$download/install/$file},
                dir  => dirname( $install->get_path( q{dev} ) ),
            );

        }

    }

    if ( $download_content->{build} ) {

        # NOTE: There is always a common build tarball, and it has nothing to
        # do with the common install.  It's the distribution file, and the
        # efsdeploy.state (if any).
        my @execs = grep { $_ ne q{common} } @installs;

        foreach my $install_name ( @execs, q{common} ) {

            my $file = qq{$install_name.tar.bz2};

            $self->download_file(
                from => qq{$download_root/build/$file},
                to   => qq{$download/build/$file},
                sum  => $metadata->{sums}->{ qq{build/$install_name} },
            );

            $self->extract_file(
                file => qq{$download/build/$file},
                dir  => $release->get_path( q{dev} ) . q{/build},
            );

        }

    }

    if ( $metadata->{depends} ) {

        foreach my $type ( sort keys %{ $self->download_depends } ) {

            next if not $metadata->{depends}->{$type};

            my $dependencies =
                join( q{,}, split( m{\s+}msx, $metadata->{depends}->{$type} ) );

            EFS::CLI::Dependency->batchdefine(
                dependent_metaproj => $metaproj_name,
                dependent_project  => $project_name,
                dependent_release  => $release_name,
                dependencies       => $dependencies,
                type               => $type,
                complete           => 1,
            );

        }

    }

    my $needdist = $download_content->{install} && scalar @installs;

    if ( $needdist ) {

        EFS::CLI::Release->checkpoint_rpc(
            history  => $self->history,
            metaproj => $metaproj_name,
            project  => $project_name,
            name     => $release_name,
        );

        EFS::CLI::Release->dist_rpc(
            history  => $self->history,
            metaproj => $metaproj_name,
            project  => $project_name,
            name     => $release_name,
            cells    => $self->cells,
        );

    }

    # We make two passes here, because we need to create:
    # x.y.z -> x.y.z-buildXXX
    # before we create
    # x.y.z-redirect -> x.y.z
    my $releasealiases = $metadata->{releasealiases};

    my %done = ();

    foreach my $name ( reverse sort keys %{ $releasealiases } ) {
        my $target = $releasealiases->{$name};
        next if length $target <= length $name;
        $self->create_dist_releasealias( $name, $target, $needdist );
        $done{$name}++;
    }

    foreach my $name ( sort keys %{ $releasealiases } ) {
        next if $done{$name};
        my $target = $releasealiases->{$name};
        $self->create_dist_releasealias( $name, $target, $needdist );
    }

    $release->set_column( downloaded => join(q{-}, Today ) );
    $release->update;

    STDOUT->print( qq{Downloaded release $release\n} );

    return 1;

}

sub destroy {

    my $self = shift;

    my $metaproj = $self->metaproj;
    my $project  = $self->project;
    my $release  = $self->_release_obj->name;

    my $upload_root = $self->upload_root;
    my $destroyed   = qq{$upload_root/$metaproj/$project/.destroyed};

    $self->remote_mkdir(
        dir  => $destroyed,
        mode => q{700},
    );

    $self->remote_move(
        from => qq{$upload_root/$metaproj/$project/$release},
        to   => $destroyed,
    );

    $self->run_upload_hook;

    my $mpr           = $self->mpr;
    my $upload_server = $self->upload_server;
    STDOUT->print( qq{Destroyed $mpr package on $upload_server\n} );
    return 1;

}

sub recover {

    my $self = shift;

    my $metaproj = $self->metaproj;
    my $project  = $self->project;
    my $release  = $self->_release_obj->name;

    my $upload_root = $self->upload_root;
    my $destroyed   = qq{$upload_root/$metaproj/$project/.destroyed/$release};

    $self->remote_move(
        from => $destroyed,
        to   => qq{$upload_root/$metaproj/$project}
    );

    $self->run_upload_hook;

    my $mpr           = $self->mpr;
    my $upload_server = $self->upload_server;
    STDOUT->print( qq{Recovered $mpr package on $upload_server\n} );
    return 1;

}

sub purge {

    my $self = shift;

    my $metaproj = $self->metaproj;
    my $project  = $self->project;
    my $release  = $self->_release_obj->name;

    my $upload_root = $self->upload_root;
    my $destroyed   = qq{$upload_root/$metaproj/$project/.destroyed/$release};

    $self->remote_rmdir( dir => $destroyed );

    $self->run_upload_hook;

    my $mpr           = $self->mpr;
    my $upload_server = $self->upload_server;
    STDOUT->print( qq{Purged $mpr package on $upload_server\n} );
    return 1;

}

sub get_metadata {

    my $self = shift;

    if ( my $project = $self->_project_obj ) {
        $project->assert_downloadable;
    }
    else {
        $self->_metaproj_obj->assert_downloadable;
    }

    my $download_url  = $self->download_url;
    my $mpr           = $self->mpr;

    my $download_mpr  = qq{$download_url/$mpr};

    my %url_of = (
        metaproj => dirname( dirname( $download_mpr ) ),
        project  => dirname( $download_mpr ),
        release  => $download_mpr,
    );

    foreach my $type ( qw( metaproj project release ) ) {
        $self->ping_url( url => $url_of{$type} )
            or die qq{No such $type for $mpr at $download_url\n};
    }

    my $file = qq{$download_url/$mpr/metadata.conf};

    my $tmpfh = File::Temp->new(
        TEMPLATE => qq{.efs_package_${PID}_XXXXXXXX},
        DIR      => q{/var/tmp},
        UNLINK   => 1,
    );

    my $data = $self->retrieve_url( url => $file );

    $data =~ s{\#.*?$}{}gmsx;
    $data =~ s{\\\n}{}gmsx;
    $data =~ s{^\s+}{}gmsx;
    $data =~ s{\s+$}{}gmsx;

    my $metadata = Config::Tiny->read_string( $data );

    if ( $Config::Tiny::errstr ) {
        die( qq{Unable to parse $file:\n} . $data );
    }

    if ( $self->metaproj ne $metadata->{global}->{metaproj} ) {
        die(
            qq{Invalid metadata file.  Metaproj names do not match\n},
            q{Want => }, $self->metaproj, qq{\n},
            qq{Have => $metadata->{global}->{metaproj}\n},
        );
    }

    if ( $self->project ne $metadata->{global}->{project} ) {
        die(
            qq{Invalid metadata file.  Project names do not match\n},
            q{Want => }, $self->project, qq{\n},
            qq{Have => $metadata->{global}->{project}\n},
        );
    }

    my $want = $self->name;
    my $have = $metadata->{global}->{release};

    if ( $want ne $have and length $want eq length $have ) {
        if ( not $metadata->{releasealiases} or
                 not $metadata->{releasealiases}->{$want} ) {
            die(
                qq{Invalid metadata file.  Release names do not match, and $want is not an alias\n},
                qq{Want => $want\n},
                qq{Have => $have\n},
            );
        }
    }
    elsif ( $have !~ m{^$want}msx ) {
        die(
            qq{Invalid metadata file.  Release names do not match\n},
            qq{Want => $want.*\n},
            qq{Have => $have\n},
        );
    }

    if ( not $metadata->{global}->{installs} ) {
        die qq{Invalid metadata file.  No installs\n};
    }

    $self->_metadata( $metadata );

    return 1;

}

sub releasealiases {

    my $self = shift;

    my $release = $self->_release_obj;
    my $release_name = $release->name;

    my $releasealiases = {};

    foreach my $releasealias ( $release->all_releasealiases ) {
        my $name = $releasealias->name;
        next if not $release_name =~ m{^$name}msx;
        $releasealiases->{ $name } = $releasealias->target->name;
    }

    # This picks up the x.y.z-redirect => x.y.z aliases
    foreach my $releasealias ( $release->all_releasealiases ) {
        my $name = $releasealias->name;
        next if not length $name eq length $release_name;
        my $target = $releasealias->target->name;
        next if not $releasealiases->{ $target };
        $releasealiases->{ $name } = $target;
    }

    return $releasealiases;

}

sub dependencies {

    my $self = shift;
    my $type = shift;
    my $metadata = $self->_metadata();

    return if not $metadata->{depends};
    return if not $metadata->{depends}->{$type};
    return split m{\s+}msx, $metadata->{depends}->{$type};

}

sub filter_installs {

    my $self     = shift;
    my @installs = @_;

    my @filtered = ();

    my $platform_rs = EFS::DBI->get_resultset( q{Platform} );
    my $instance_rs = EFS::DBI->get_resultset( q{PlatformInstance} );

    foreach my $name ( @installs ) {

        if ( $name eq q{common} ) {
            push @filtered, $name;
        }
        else {
            my ($platform) = $platform_rs->search({ name => $name });
            my ($instance) = $instance_rs->search({ name => $name });

            if ( not $platform and not $instance ) {
                STDOUT->print( qq{Skipping unsupported platform '$name'\n} );
            }
            else {
                push @filtered, $name;
            }

        }

    }

    return @filtered;

}

sub download_file {

    my $self = shift;
    my %args = @_;

    my $from    = $args{from};
    my $to      = $args{to};
    my $wantsum = $args{sum};

    if ( -f $to ) {
        unlink $to || die qq{Unable to unlink $to: $ERRNO\n};
    }

    STDOUT->print( qq{Downloading $from\n} );

    my $to_dir = dirname $to;

    $self->retrieve_url( url => $from, save_file => $to );

    return if not $wantsum;

    my $havesum = $self->sha1sum($to);
    if ( $havesum ne $wantsum ) {
        die(
            qq{Invalid checksum for $to:\n},
            qq{Want => $wantsum\n},
            qq{Have => $havesum\n},
        );
    }

    return 1;

}

sub extract_file {
    my $self = shift;
    my %args = @_;
    system qq{cd $args{dir} && tar jxf $args{file}};
    $CHILD_ERROR and die qq{Unable to extract $args{file} to $args{dir}\n};
    return 1;
}

sub package_dir {
    my $self = shift;
    my %args = @_;
    my ($name,$dir) = fileparse( $args{dir} );
    STDOUT->print( qq{Creating $args{file}\n} );
    system qq{cd $dir && tar jcf $args{file} $name};
    $CHILD_ERROR and die qq{Unable to package $args{dir} into $args{file}\n};
    return 1;
}

sub create_dist_releasealias {

    my $self   = shift;

    my $name     = shift;
    my $target   = shift;
    my $needdist = shift;

    my $metadata = $self->_metadata;

    my $metaproj_name = $metadata->{global}->{metaproj};
    my $project_name  = $metadata->{global}->{project};

    EFS::CLI::ReleaseAlias->create_rpc(
        history  => $self->history,
        metaproj => $metaproj_name,
        project  => $project_name,
        name     => $name,
        target   => $target,
    );

    return if not $needdist;

    EFS::CLI::ReleaseAlias->dist_rpc(
        history  => $self->history,
        metaproj => $metaproj_name,
        project  => $project_name,
        name     => $name,
        cells    => $self->cells,
    );

    return 1;

}

sub ssh {
    my $self    = shift;
    my $command = shift;
    my $ssh           = $self->ssh_command;
    my $upload_server = $self->upload_server;
    my $errors = qx{$ssh $upload_server $command 2>&1};
    $CHILD_ERROR and die qq{Error running $ssh $upload_server $command\n$errors};
    return 1;
}

sub ssh_command {
    my $self = shift;
    my $upload_sshkey   = $self->upload_sshkey;
    my $upload_username = $self->upload_username;
    return qq{ssh -o StrictHostKeyChecking=no -i $upload_sshkey -l $upload_username};
}

sub rsync {
    my $self = shift;
    my %args = @_;
    my $ssh = $self->ssh_command;
    my $rsync = qq{rsync --rsh='$ssh' --archive --verbose};
    $rsync .= q{ --delete} if $args{delete};
    my $upload_server = $self->upload_server;
    my $errors = qx{$rsync $args{from} $upload_server:$args{to} 2>&1};
    $CHILD_ERROR and die qq{Error running $rsync $args{from} $upload_server:$args{to}\n$errors\n};
    return 1;
}

sub run_upload_hook {
    my $self = shift;
    return if not $self->upload_hook;
    return $self->ssh( $self->upload_hook );
}

sub remote_move {
    my $self = shift;
    my %args = @_;
    return $self->ssh( qq{mv $args{from} $args{to}} );
}

sub remote_rmdir {
    my $self = shift;
    my %args = @_;
    return $self->ssh( qq{rm -rf $args{dir}} );
}

sub remote_mkdir {
    my $self = shift;
    my %args = @_;
    $self->ssh( qq{mkdir -p $args{dir}} );
    return if not $args{mode};
    return $self->ssh( qq{chmod $args{mode} $args{dir}} );
}

sub _migrate_build_common {

    my $self = shift;
    my $release = shift;

    my $build = $release->get_path( q{dev} ) . q{/build};

    EFS::Util->MakeDir( path => qq{$build/common} );

    if ( -f qq{$build/efsdeploy.state} ) {
        rename( qq{$build/efsdeploy.state},  qq{$build/common/efsdeploy.state} ) or
            die qq{Unable to rename $build/efsdeploy.state: $ERRNO\n};
        STDOUT->print( qq{Moved efsdeploy.state to $build/common\n} );
    }

    if ( -d qq{$build/download} ) {

        my $io = IO::Dir->new( qq{$build/download} )
            or die qq{Unable to opendir $build/download: $ERRNO\n};

        foreach my $file ( $io->read ) {
            next if not -f qq{$build/download/$file};
            next if $file ~~ [qw( metadata.conf src.tar.bz2 )];
            rename( qq{$build/download/$file}, qq{$build/common/$file} ) or
                die qq{Unable to rename $build/download/$file: $ERRNO\n};
            STDOUT->print( qq{Moved download/$file to $build/common\n} );
        }

    }

    return 1;

}

sub retrieve_url {
    my $self = shift;
    my %args = @_;

    my $request = HTTP::Request->new( GET => $args{url} );
    my $response;

    if ( $args{save_file} ) {
        $response = $self->_lwp_ua()->request( $request, $args{save_file} );
    }
    else {
        $response = $self->_lwp_ua()->request( $request );
    }

    if ( ! $response->is_success() ) {
        my $code = $response->code();
        my $message = $response->message();
        my $err = qq{Could not download url [$args{url}]};
        if ( $args{save_file} ) {
            $err .= qq{ to [$args{save_file}]};
        }
        $err .= qq{: $code $message\n};
        die $err;
    }

    return $args{save_file} ? 1 : $response->content();
}

sub sha1sum {
    my $self = shift;
    my $file = shift;
    my $sum  = qx{sha1sum $file};
    $CHILD_ERROR and die qq{Unable to checksum $file\n};
    my @sum = split qr{\s+}, $sum;
    return $sum[0];
}

# returns 1 for successful ping, 0 otherwise
sub ping_url {
    my $self = shift;
    my %args = @_;

    my $request = HTTP::Request->new( HEAD => $args{url} );
    my $response = $self->_lwp_ua()->request( $request );

    return $response->is_success() ? 1 : 0;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;

# Local Variables:
#   mode: cperl
#   cperl-indent-level: 4
#   fill-column: 78
# End:
# vim: expandtab shiftwidth=4:
