#!/usr/bin/env perl

## no critic: RequireExtendedFormatting

use strict;
use warnings;
use English;

use Test::More tests => 49;

use lib q{test/lib};
use EFS::Test;

my $test   = EFS::Test->new;
my $config = $test->get_config;

my $username  = $test->get_primary_username;
my $groupname = $test->get_primary_groupname;

my $ldc_name      = $config->get_value( q{LocalDevCell}, q{name} );
my $ldc_shortname = $config->get_value( q{LocalDevCell}, q{shortname} );

my $rdc_name      = $config->get_value( q{RemoteDevCell}, q{name} );
my $rdc_shortname = $config->get_value( q{RemoteDevCell}, q{shortname} );

my @setup = (
    q{efs display metaproj testmeta},
    q{efs display project testmeta testproj},
    q{efs create release testmeta testproj testrel},
);

foreach my $setup (@setup) {
    $test->efs_test_command(
        name    => qq{(setup) $setup},
        command => $setup,
        bail    => 1,
    );
}

$test->efs_test_command(
    command => q{efs display feature enable_releaselink_compatibility},
    not     => 1,
    expect  => q{No such feature 'enable_releaselink_compatibility'},
    bail    => 1,
);

$test->efs_test_command(
    command => q{efs create releaselink testmeta testproj compatibility testrel},
    not     => 1,
    expect  => q{Releaselink has been renamed to releasealias},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs define feature enable_releaselink_compatibility true},
);

$test->efs_test_command(
    command => q{efs create releaselink testmeta testproj compatibility testrel},
    filter  => qr{ renamed }msx,
    expect  => q{WARNING: Releaselink has been renamed to releasealias},
);

$test->efs_test_command(
    command => q{efs destroy releaselink testmeta testproj compatibility},
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => q{efs undef feature enable_releaselink_compatibility},
);

$test->efs_test_command(
    command => q{efs setstage release testmeta testproj testrel rejected},
    expect  => q{Changed the stage of testmeta/testproj/testrel from dev to rejected},
);

$test->efs_test_command(
    name    => q{create a releasealias (to rejected release - should fail)},
    command => q{efs create releasealias testmeta testproj rlink2release testrel},
    not     => 1,
    expect  => [
        q{The target release 'testmeta/testproj/testrel' is in stage 'rejected'},
        q{Release aliases may not resolve to rejected releases.},
    ],
);

$test->efs_test_command(
    name    => q{setstage back to dev},
    command => q{efs setstage release testmeta testproj testrel dev},
    expect  => q{Changed the stage of testmeta/testproj/testrel from rejected to dev},
);

$test->efs_test_command(
    name    => q{create releasealias},
    command => q{efs create releasealias testmeta testproj rlink2release testrel},
    expect  => [
        q{Created symlink /efs/dev/testmeta/testproj/rlink2release -> testrel},
        q{Created releasealias testmeta/testproj/rlink2release -> testrel},
    ],
);

$test->efs_test_command(
    name    => qq{dist to $ldc_shortname - should fail, no installs },
    command => qq{efs dist releasealias testmeta testproj rlink2release -cell $ldc_shortname},
    not     => 1,
    expect  => [
        q{Cannot dist releasealias because the target release has no},
        q{installs, and hence no diststate.  Create installs for the},
        q{target and dist the release before proceeding.},
    ],
);

$test->efs_test_command(
    name    => q{create an install},
    command => q{efs create install testmeta testproj testrel common},
    expect  => [
        qq{Created directory $ldc_name:/efs/dev/testmeta/testproj/testrel/install/common},
        qq{Changed ownership of $ldc_name:/efs/dev/testmeta/testproj/testrel/install/common to $username/$groupname},
        q{Created install /efs/dev/testmeta/testproj/testrel/install/common},
    ],
);

$test->efs_test_command(
    name    => qq{dist to $ldc_shortname - should fail, undisted target },
    command => qq{efs dist releasealias testmeta testproj rlink2release -cell $ldc_shortname},
    not     => 1,
    expect  => [
        q{The target release(link) has not been disted anywhere yet.},
        q{Dist the target object before disting this releasealias.},
    ],
);

$test->efs_test_command(
    command => q{touch /efs/dev/testmeta/testproj/testrel/install/common/not-empty},
);

$test->efs_test_command(
    command => q{efs checkpoint release testmeta testproj testrel},
);

$test->efs_test_command(
    name    => qq{dist target to $ldc_shortname},
    command => qq{efs dist release testmeta testproj testrel -cells $ldc_shortname},
);

$test->efs_test_command(
    ## no critic: RequireExtendedFormatting
    name    => qq{dist to $ldc_shortname - should pass now},
    command => qq{efs dist releasealias testmeta testproj rlink2release -cell $ldc_shortname},
    filter  => qr{ \A (?:Created \s+ symlink|Distributed|Successfully) }msx,
    expect  => [
        qr{Created symlink $ldc_name:/efs/dist/testmeta/testproj/rlink2release -> testrel}ms,
        qr{Distributed \S+ testmeta/testproj/rlink2release -> testrel to $ldc_name}ms,
        qr{Successfully distributed releasealias testmeta/testproj/rlink2release to all cells}ms,
    ],
);

$test->efs_test_command(
    name    => qq{dist to $rdc_shortname - should fail, undisted target },
    command => qq{efs dist releasealias testmeta testproj rlink2release -cell $rdc_shortname},
    not     => 1,
    expect  => [
        q{Diststate for target release testmeta/testproj/testrel:},
        q{Dev environment diststate:},
        qr{Install \s+ $rdc_shortname}xms,
        qr{^=+}msx,
        qr{common \s+ X}xms,
        qr{^-+}msx,
        q{Legend:},
        q{D: Item is disted and up to date},
        q{P: Item is disted but needs update},
        q{X: Item has not been disted},
        q{Unable to dist releasealias because the target:},
        qr{testmeta/testproj/testrel}msx,
        q{is not disted to all requested cells.  Dist the target to those},
        q{cells before proceeding.},
    ],
);

$test->efs_test_command(
    name    => qq{dist target to $rdc_shortname},
    command => qq{efs dist release testmeta testproj testrel -cells $rdc_shortname},
    bail    => 1,
);

$test->efs_test_command(
    name    => qq{dist to $rdc_shortname - should pass now},
    command => qq{efs dist releasealias testmeta testproj rlink2release -cell $rdc_shortname},
    filter  => qr{ \A (?:Created \s+ symlink|Distributed|Successfully) }msx,
    expect  => [
        qr{Created symlink $rdc_name:/efs/dist/testmeta/testproj/rlink2release -> testrel}ms,
        qr{Distributed \S+ testmeta/testproj/rlink2release -> testrel to $rdc_name}ms,
        q{Successfully distributed releasealias testmeta/testproj/rlink2release to all cells},
    ],
);

$test->efs_test_link(
    name   => q{check symlink and target},
    link   => q{/efs/dist/testmeta/testproj/rlink2release},
    target => q{testrel},
);

$test->efs_test_command(
    name    => q{create releasealias to releasealias},
    command => q{efs create releasealias testmeta testproj rlink2rlink rlink2release},
    filter  => qr{Created}msx,
    expect  => [
        q{Created symlink /efs/dev/testmeta/testproj/rlink2rlink -> rlink2release},
        q{Created releasealias testmeta/testproj/rlink2rlink -> rlink2release},
    ],
);

## Test "efs list" commands
$test->efs_test_command(
    name    => q{"efs list any" returns the metaproj},
    command => q{efs list any},
    filter  => qr{testmeta\z}xms,
    expect  => q{testmeta},
);

$test->efs_test_command(
    name    => q{"efs list any -devpaths" works},
    command => q{efs list any -devpaths},
    filter  => qr{/ testmeta \z}xms,
    expect  => q{/efs/dev/testmeta},
);

$test->efs_test_command(
    name    => q{"efs list metaproj" returns the project},
    command => q{efs list metaproj testmeta},
    expect  => q{testproj},
);

$test->efs_test_command(
    name    => q{"efs list metaproj -devpaths" works},
    command => q{efs list metaproj testmeta -devpaths},
    expect  => q{/efs/dev/testmeta/testproj},
);

# Future-proofing in case value of testrel changes later: we stuff it
# into a list and sort it with the rlinks
my @rels_with_showlinks = (
    q{rlink2release@ -> testrel},
    q{rlink2rlink@ -> rlink2release},
    q{testrel},
);
my @rels = @rels_with_showlinks;
for (@rels) {
    s/ \@ .* //xms;
}

my @paths = map { q{/efs/dev/testmeta/testproj/} . $_ } @rels;
my @paths_with_showlinks = map { q{/efs/dev/testmeta/testproj/} . $_ } @rels_with_showlinks;

$test->efs_test_command(
    name    => q{"efs list project" returns releases and releasealiases},
    command => q{efs list project testmeta testproj},
    expect  => [@rels],
);

$test->efs_test_command(
    name    => q{"efs list project -showlinks" works},
    command => q{efs list project testmeta testproj -showlinks},
    expect  => [@rels_with_showlinks],
);

$test->efs_test_command(
    name    => q{"efs list project -devpaths" works},
    command => q{efs list project testmeta testproj -devpaths},
    expect  => [@paths],
);

$test->efs_test_command(
    name    => q{"efs list project -devpaths -showlinks" works},
    command => q{efs list project testmeta testproj -devpaths -showlinks},
    expect  => [@paths_with_showlinks],
);
## end of "efs list" test block (but one more below after destroy)

$test->efs_test_command(
    name    => q{destroy first releasealias (should fail)},
    command => q{efs destroy releasealias testmeta testproj rlink2release},
    not     => 1,
    filter  => qr{The \s+ following|this \s+ releasealias|rlink2rlink}msx,
    expect  => [
        q{The following releasealiases must be destroyed or changed before},
        q{this releasealias can be destroyed:},
        q{testmeta/testproj/rlink2rlink},
    ],
);

$test->efs_test_command(
    name    => q{destroy second releasealias},
    command => q{efs destroy releasealias testmeta testproj rlink2rlink},
    filter  => qr{destroyed}imsx,
    expect  => [
        q{Destroyed 'testmeta/testproj/rlink2rlink' in /efs/dev.},
        q{Successfully destroyed releasealias testmeta/testproj/rlink2rlink in all cells},
    ],
);

@rels = grep { $_ ne q{rlink2rlink} } @rels;

$test->efs_test_command(
    name    => q{"efs list project" does not return destroyed releasealias 1},
    command => q{efs list project testmeta testproj},
    expect  => [@rels],
);

$test->efs_test_command(
    name    => q{destroy first releasealias},
    command => q{efs destroy releasealias testmeta testproj rlink2release},
);

@rels = grep { $_ ne q{rlink2release} } @rels;

$test->efs_test_command(
    name    => q{"efs list project" does not return destroyed releasealias 2},
    command => q{efs list project testmeta testproj},
    expect  => [@rels],
);

my $newrel = q{listtest};

$test->efs_test_command(
    name    => q{Creating a new release temporarily to test efs list},
    command => qq{efs create release testmeta testproj $newrel},
);

$test->efs_test_command(
    name    => q{"efs list project" returns new release},
    command => q{efs list project testmeta testproj},
    expect  => [ sort ( @rels, $newrel ) ],
);

$test->efs_test_command(
    name    => q{Destroying temporary release to test efs list},
    command => qq{efs setstage release testmeta testproj $newrel rejected && }
        . qq{efs destroy release testmeta testproj $newrel }
);

$test->efs_test_command(
    name    => q{"efs list project" does not return destroyed release},
    command => q{efs list project testmeta testproj},
    expect  => [@rels],
);

$test->efs_test_command(
    name    => q{"efs list project -all" returns destroyed release},
    command => q{efs list project testmeta testproj -all},
    expect  => [ sort ( @rels, $newrel ) ],
);

$test->efs_test_command(
    name    => q{"efs list project -all -showlinks" works},
    command => q{efs list project testmeta testproj -all -showlinks},
    expect  => [ sort ( @rels, qq{$newrel (destroyed)} ) ],
);

$test->efs_test_command(
    name    => q{"efs list project -all -devpaths" fails},
    command => q{efs list project testmeta testproj -all -devpaths},
    not     => 1,
);

$test->efs_test_command(
    name    => q{Purge test release},
    command => qq{efs purge release testmeta testproj $newrel -now},
);

my @teardown = (
    q{efs setstage release testmeta testproj testrel rejected},
    q{efs destroy release testmeta testproj testrel},
    q{efs purge release testmeta testproj testrel -now},
);

foreach my $teardown (@teardown) {
    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

exit 0;
