#!/usr/bin/env perl

use strict;
use warnings;
use English;

use Test::More tests => 33;

use lib q{test/lib};
use EFS::Test;

my $test   = EFS::Test->new;
my $config = $test->get_config;

my $username = $test->get_primary_username;

my $root_domain     = $config->get_value( q{Global}, q{root_domain} );
my $secondary_user  = $config->get_value( q{Global}, q{secondary_user} );
my $secondary_group = $config->get_value( q{Global}, q{secondary_group} );

my $ldc_name      = $config->get_value( q{LocalDevCell}, q{name} );
my $ldc_shortname = $config->get_value( q{LocalDevCell}, q{shortname} );

my $rdc_name      = $config->get_value( q{RemoteDevCell}, q{name} );
my $rdc_shortname = $config->get_value( q{RemoteDevCell}, q{shortname} );

my $ownerlist = join q{@}, $username, $root_domain;

my $mp_name = q{testmeta testdisttree};
my $mp_path = q{testmeta/testdisttree};

my @setup = (
    [ q{primary_user},
      q{efs display metaproj testmeta} ],
    [ q{efsops},
      qq{efs create metaproj othermeta $secondary_user $secondary_group $ownerlist} ],
);

foreach my $metaproj ( qw( testmeta othermeta ) ) {
    push @setup, (
        [ q{efsops},
          qq{efs create project $metaproj testdisttree $ldc_name} ],
        [ q{efsops},
          qq{efs create release $metaproj testdisttree testrel} ],
        [ q{efsops},
          qq{efs create install $metaproj testdisttree testrel common} ],
        [ q{root},
          qq{touch /efs/dev/$metaproj/testdisttree/testrel/install/common/foo} ],
        [ q{efsops},
          qq{efs checkpoint release $metaproj testdisttree testrel} ],
    );
}

push @setup,
    [ q{primary_user},
      q{efs define dependency testmeta testdisttree testrel } .
          q{othermeta testdisttree testrel runtime} ];

foreach my $setup (@setup) {

    my ($run_as,$command) = @{ $setup };

    $test->efs_test_command(
        run_as  => $run_as,
        name    => qq{(setup) $command},
        command => $command,
        bail    => 1,
    );
}

$test->efs_test_command(
    command => q{efs disttree release testmeta testdisttree testrel --global},
    not     => 1,
    expect  => [
        q{Access denied.},
        q{Insufficient access to disttree dependencies:},
        q{The 'dev' role is required to }.
            q{disttree 'othermeta/testdisttree/testrel'},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs define userrights othermeta dev $username},
);

$test->efs_test_command(
    command => q{efs disttree release testmeta testdisttree testrel --global},
);

foreach my $metaproj ( qw( othermeta testmeta ) ) {
    $test->efs_test_command(
        command => qq{efs setstage release $metaproj testdisttree testrel prod},
    );
}

$test->efs_test_command(
    command => qq{efs define userrights testmeta prod $username},
);

$test->efs_test_command(
    command => q{efs disttree release testmeta testdisttree testrel --celltype prod --global},
    not     => 1,
    expect  => [
        q{Access denied.},
        q{Insufficient access to disttree dependencies:},
        q{The 'prod' role is required to }.
            q{disttree 'othermeta/testdisttree/testrel'},
    ],
);

$test->efs_test_command(
    run_as  => q{efsops},
    command => qq{efs define userrights othermeta admin $username},
);

$test->efs_test_command(
    command => qq{efs define userrights othermeta prod $username},
);

$test->efs_test_command(
    command => q{efs disttree release testmeta testdisttree testrel --celltype prod --global},
);

my @teardown = (
    q{efs undef dependency testmeta testdisttree testrel } .
        q{othermeta testdisttree testrel runtime},
);

foreach my $metaproj ( qw( testmeta othermeta ) ) {
    push @teardown, (
        qq{efs setstage release $metaproj testdisttree testrel deprecated},
        qq{efs destroy release $metaproj testdisttree testrel},
        qq{efs purge release $metaproj testdisttree testrel -now},
        qq{efs destroy project $metaproj testdisttree},
    );
}

push @teardown, q{efs destroy metaproj othermeta};

foreach my $teardown (@teardown) {
    $test->efs_test_command(
        run_as  => q{efsops},
        name    => qq{(teardown) $teardown},
        command => $teardown,
    );
}

exit 0;

