#!/usr/bin/env perl

use strict;
use warnings;
use English;

use blib;
use EFS;

use lib q{test/lib};
use EFS::Test;
use EFS::Test::Config;

use List::MoreUtils qw( all );
use Test::More tests => 19;

my $test   = EFS::Test->new( as_user => q{efsops} );
my $config = EFS::Test::Config->new;

my $ldc_efsserver = $config->get_value( q{LocalDevCell}, q{efsserver} );
my $rdc_efsserver = $config->get_value( q{RemoteDevCell}, q{efsserver} );

my @before = qx{ find $EFS::logroot -type f -print };
ok( $CHILD_ERROR == 0, q{Got list of files before archiving} );

chomp @before;
my %before = map { $_ => 1 } @before;

my @output = ();

$test->efs_test_command(
    command => q{efs archivelogs efsserver},
    output  => \@output,
);

my $no_remote_logs = 0;

my $parse_errors = q{};

my %archived = ();
my %unlinked = ();
my %ignored  = ();

foreach my $output ( @output ) {

    chomp $output;

    if ( $output eq qq{$EFS::logroot on $rdc_efsserver does not contain any logfiles.} ) {
        $no_remote_logs = 1;
        next;
    }

    if ( my ($efshost_path,$archive_path) =
             ($output =~ /Archived \s+ (\S+) \s+ -> \s+ (\S+)/msx) ) {

        my ($efshost,$path) = split /:/msx, $efshost_path;

        if ( $efshost ne $ldc_efsserver ) {
            $parse_errors .= qq{# Bogus hostname: $output\n};
        }

        if ( not $before{$path} ) {
            $parse_errors .= qq{# Bogus log path: $output\n};
        }

        if ( not -f $archive_path ) {
            $parse_errors .= qq{# Bogus archive path: $output\n};
        }

        $archived{$path} = $archive_path;

        next;

    }

    if ( $output =~ /Skipping\s+archive\s+of\s+active\s+logfile\s+(\S+)/msx ) {
        $ignored{$1}++;
        next;
    }

    if ( $output =~ /No\s+history\s+entries\s+found\s+for\s+logfile\s+(\S+)/msx ) {
        $unlinked{$1}++;
        next;
    }

}

ok( $parse_errors eq q{}, q{No errors parsing output} );

if ( $parse_errors ) {
    STDOUT->print($parse_errors);
}

ok( $no_remote_logs, q{No remote logs archived} );

my @after = qx{ find $EFS::logroot -type f -print };
ok( $CHILD_ERROR == 0, q{Got list of files after archiving} );

chomp @after;
my %after = map { $_ => 1 } @after;

if ( all { not -f $_ } sort keys %archived ) {
    ok( 1, q{Logfile paths no longer found locally} );
}
else {
    ok( 0, q{Logfile paths no longer found locally} );
    STDOUT->print(
        qq{# The following paths were not archived:\n},
        map { qq{# $_\n} } grep { -f $_ } sort keys %archived,
    );
}

if ( all { -f $_ } sort values %archived ) {
    ok( 1, q{Logfile paths all found in archive} );
}
else {
    ok( 0, q{Logfile paths all found in archive} );
    STDOUT->print(
        qq{# The following paths were not found in the archive:\n},
        map { qq{# $_\n} } grep { not -f $_ } sort values %archived,
    );
}

if ( all { -f $_ } sort keys %ignored ) {
    ok( 1, q{Skipped logfiles still present} );
}
else {
    ok( 0, q{Skipped logfiles still present} );
    STDOUT->print(
        qq{# The following paths should have been ignored, but are missing:\n},
        map { qq{# $_\n} } grep { not -f $_ } sort keys %ignored,
    );
}

# NOTE: We check %unlinked first, because there won't be any unlinked files
# if you run this test standalone, when debugging it.
if ( not %unlinked or all { not -f $_ } sort keys %unlinked ) {
    ok( 1, q{Unlinked logfiles gone} );
}
else {
    ok( 0, q{Unlinked logfiles gone} );
    STDOUT->print(
        qq{# The following paths should have been unlinked, but weren't:\n},
        map { qq{# $_\n} } grep { -f $_ } sort keys %unlinked,
    );
}

if ( all { $ignored{$_} } sort keys %after ) {
    ok( 1, q{Leftover logfiles accounted for} );
}
else {
    ok( 0, q{Leftover logfiles accounted for} );
    STDOUT->print(
        qq{# The following loftover logfiles should have been ignored:\n},
        map { qq{# $_\n} } grep { not $ignored{$_} } sort keys %after,
    );
}

my $before_count   = scalar keys %before;
my $archive_count  = scalar keys %archived;
my $unlinked_count = scalar keys %unlinked;
my $after_count    = scalar keys %after;

if ( $before_count = $archive_count + $unlinked_count + $after_count ) {
    ok( 1, q{All file accounting balances} );
}
else {
    ok( 0, q{All file accounting balances} );
    STDOUT->print(
        qq{# File count before archiving: $before_count\n},
        qq{# Archived files: $archive_count\n},
        qq{# Unlinked files: $unlinked_count\n},
        qq{# Remaining files: $after_count\n},
        qq{# $before_count != $archive_count + $unlinked_count + $after_count\n},
    );
}

# We know there should be ONE logfile left locally, the one for the
# archivelogs command itself, which it ignored.

ok( $after_count == 1, qq{Only one logfile left in $EFS::logroot} );

my ($logfile) = keys %after;

my @history_before = ();
$test->efs_test_command(
    command => q{efs history any -command 'archivelogs efsserver' -count 1 -verbose},
    output  => \@history_before,
);

my $history = $history_before[2];
$history =~ s/^\s+//msx;

my ($history_id,$history_logfile) = (split /\s+/msx, $history)[0,10];

ok( -f $history_logfile, q{Skipped logfile still present} );

my @logfile_before = ();

$test->efs_test_command(
    command => qq{efs history logfile $history_id},
    output => \@logfile_before,
);

$test->efs_test_command(
    command => q{efs archivelogs efsserver},
);

ok( (not -f $history_logfile), q{Previously skipped logfile was archived} );

my @logfile_after = ();

$test->efs_test_command(
    command => qq{efs history logfile $history_id},
    output => \@logfile_after,
);

ok( join( q{}, @logfile_before ) eq join( q{}, @logfile_after ),
    q{Same output from history logfile before/after archiving} );

